@@ -488,6 +488,9 @@ soup_connection_connect_sync (SoupConnection *conn)
 	if (!SOUP_STATUS_IS_SUCCESSFUL (status))
 		goto fail;
 
+	g_signal_connect (conn->priv->socket, "disconnected",
+			  G_CALLBACK (socket_disconnected), conn);
+
 	if (conn->priv->conn_uri->protocol == SOUP_PROTOCOL_HTTPS) {
 		if (!soup_socket_start_ssl (conn->priv->socket)) {
 			status = SOUP_STATUS_SSL_FAILED;
@@ -527,9 +530,9 @@ soup_connection_connect_sync (SoupConnection *conn)
 		}
 	}
 
-	g_signal_emit (conn, signals[CONNECT_RESULT], 0,
-		       proxified_status (conn, status));
-	return proxified_status (conn, status);
+	status = proxified_status (conn, status);
+	g_signal_emit (conn, signals[CONNECT_RESULT], 0, status);
+	return status;
 }
 
 
@@ -601,7 +604,10 @@ soup_connection_disconnect (SoupConnection *conn)
  * soup_connection_is_in_use:
  * @conn: a connection
  *
- * Return value: whether or not @conn is being used.
+ * Tests whether or not @conn is in use.
+ *
+ * Return value: %TRUE if there is currently a request being processed
+ * on @conn.
  **/
 gboolean
 soup_connection_is_in_use (SoupConnection *conn)
@@ -615,6 +621,8 @@ soup_connection_is_in_use (SoupConnection *conn)
  * soup_connection_last_used:
  * @conn: a #SoupConnection.
  *
+ * Returns the last time a response was received on @conn.
+ *
  * Return value: the last time a response was received on @conn, or 0
  * if @conn has not been used yet.
  */
@@ -670,6 +678,14 @@ send_request (SoupConnection *conn, SoupMessage *req)
 				   conn->priv->proxy_uri != NULL);
 }
 
+/**
+ * soup_connection_send_request:
+ * @conn: a #SoupConnection
+ * @req: a #SoupMessage
+ *
+ * Sends @req on @conn. This is a low-level function, intended for use
+ * by #SoupSession.
+ **/
 void
 soup_connection_send_request (SoupConnection *conn, SoupMessage *req)
 {
@@ -680,6 +696,15 @@ soup_connection_send_request (SoupConnection *conn, SoupMessage *req)
 	SOUP_CONNECTION_GET_CLASS (conn)->send_request (conn, req);
 }
 
+/**
+ * soup_connection_reserve:
+ * @conn: a #SoupConnection
+ *
+ * Marks @conn as "in use" despite not actually having a message on
+ * it. This is used by #SoupSession to keep it from accidentally
+ * trying to queue two messages on the same connection from different
+ * threads at the same time.
+ **/
 void
 soup_connection_reserve (SoupConnection *conn)
 {
@@ -688,6 +713,36 @@ soup_connection_reserve (SoupConnection *conn)
 	conn->priv->in_use = TRUE;
 }
 
+/**
+ * soup_connection_release:
+ * @conn: a #SoupConnection
+ *
+ * Marks @conn as not "in use". This can be used to cancel the effect
+ * of a soup_session_reserve(). It is not necessary to call this
+ * after soup_connection_send_request().
+ **/
+void
+soup_connection_release (SoupConnection *conn)
+{
+	g_return_if_fail (SOUP_IS_CONNECTION (conn));
+
+	conn->priv->in_use = FALSE;
+}
+
+/**
+ * soup_connection_authenticate:
+ * @conn: a #SoupConnection
+ * @msg: the message to authenticate
+ * @auth_type: type of authentication to use
+ * @auth_realm: authentication realm
+ * @username: on successful return, will contain the username to
+ * authenticate with
+ * @password: on successful return, will contain the password to
+ * authenticate with
+ *
+ * Emits the %authenticate signal on @conn. For use by #SoupConnection
+ * subclasses.
+ **/
 void
 soup_connection_authenticate (SoupConnection *conn, SoupMessage *msg,
 			      const char *auth_type, const char *auth_realm,
@@ -697,6 +752,20 @@ soup_connection_authenticate (SoupConnection *conn, SoupMessage *msg,
 		       msg, auth_type, auth_realm, username, password);
 }
 
+/**
+ * soup_connection_reauthenticate:
+ * @conn: a #SoupConnection
+ * @msg: the message to authenticate
+ * @auth_type: type of authentication to use
+ * @auth_realm: authentication realm
+ * @username: on successful return, will contain the username to
+ * authenticate with
+ * @password: on successful return, will contain the password to
+ * authenticate with
+ *
+ * Emits the %reauthenticate signal on @conn. For use by
+ * #SoupConnection subclasses.
+ **/
 void
 soup_connection_reauthenticate (SoupConnection *conn, SoupMessage *msg,
 				const char *auth_type, const char *auth_realm,
